@@ -12,19 +12,35 @@ import functools
 import itertools
 import json
 import types
-from typing import Any, TypeVar, Union, Callable, Optional
+from typing import Any, TypeVar, Union, Callable, Optional, Protocol, ClassVar, Dict
 
 from .is_instance import get_container_type, is_instance
 from .str_to_bool import to_bool
 
 
+class IsDataclass(Protocol):
+    __dataclass_fields__: ClassVar[Dict]
+
+
 @functools.cache
-def get_dataclass_field_name_to_field(dataclass: Any) -> dict[str, dataclasses.Field]:
-    if dataclasses.is_dataclass(dataclass):
-        # dataclasses.fields does not return init only attributes
+def get_dataclass_field_name_to_field(
+        dataclass_: IsDataclass,
+        with_init_vars: bool = True, with_class_vars: bool = False
+) -> dict[str, dataclasses.Field]:
+    try:
         # noinspection PyUnresolvedReferences,PyProtectedMember
-        return getattr(dataclass, dataclasses._FIELDS)
-    raise TypeError("expected dataclass type")
+        fields = getattr(dataclass_, dataclasses._FIELDS)
+    except AttributeError:
+        raise TypeError("expected dataclass type")
+    # noinspection PyUnresolvedReferences,PyProtectedMember
+    allowed_field_types = {dataclasses._FIELD}
+    # noinspection PyUnresolvedReferences,PyProtectedMember
+    allowed_field_types |= {dataclasses._FIELD_INITVAR} if with_init_vars else set()
+    # noinspection PyUnresolvedReferences,PyProtectedMember
+    allowed_field_types |= {dataclasses._FIELD_CLASSVAR} if with_class_vars else set()
+    # noinspection PyProtectedMember
+    result = {name: field for name, field in fields.items() if field._field_type in allowed_field_types}
+    return result
 
 
 class FieldErrors(ValueError):
